#!/usr/bin/python -u
#
# Client code for Update Agent
# Copyright (c) 1999, 2000 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>

import os,sys
import string
import re
import fnmatch, glob
import rpm
import socket
import xmlrpclib, cgiwrap

sys.path.append("/usr/share/rhn/up2date")

from translate import _, N_, cat

import config

cfg = config.Up2dateConfig()
cfg.load()

def version():
    # substituted to the real version by the Makefile at installation time.
    return "2.1.7"

def hasSSL():
    return hasattr(socket, "ssl")

if cfg.readEntry("enableProxy"):
    proxyHost = cfg.readEntry("pkgProxy")
else:
    proxyHost = None

if hasSSL():
    serverUrl = cfg.readEntry("serverURL")
else:
    serverUrl = cfg.readEntry("noSSLServerURL")

# get around cached dependencies.  Gross ugly hack.
rpm.addMacro("_dbi_tags","Packages:Name:Basenames:Group:Requirename:Providename:Conflictname:Triggername")

headerCache = {}


class Error:
    # base class for client errors
    def __init__(self, errmsg):
        self.errmsg = errmsg

    def __repr__(self):
        return self.errmsg
    
class FileError(Error):
    def __repr__(self):
        return _("Disk error.  The message was:\n") + self.errmsg

class RpmError(Error):
    def __repr__(self):
        return _("RPM error.  The message was:\n") + self.errmsg

class ConflictError(Error):
    def __repr__(self):
        return _("RPM conflict error.  The message was:\n") + self.errmsg

class DependencyError(Error):
    def __init__(self, errmsg, pkgName):
	self.errmsg = errmsg
	self.pkgName = pkgName

    def __repr__(self):
        return _("RPM dependency error.  The message was:\n") + self.errmsg

class CommunicationError(Error):
    # indicates a problem doing xmlrpc communication
    def __repr__(self):
        return _("Error communicating with server.  The message was:\n") + self.errmsg


def getArch():
    s = os.uname()[4]
    return s

# return a glob for your particular architecture.
def archGlob():
    if re.search("i.86", os.uname()[4]):
        return "i?86"
    elif re.search("sparc", os.uname()[4]):
        return "sparc*"
    else:
        return os.uname()[4]

def getVersion():
    ret = ""

    if cfg.readEntry("versionOverride") != "":
        return cfg.readEntry("versionOverride")
    
    db = rpm.opendb(0, "/")
    index = db.findbyname("redhat-release")[0]
    ret = (db[index])['version']
    return ret

def getSystemId():
    path = cfg.readEntry("systemIdPath")
    if not os.access(path, os.R_OK):
        return None
    
    f = open(path, "r")
    ret = f.read()
        
    f.close()
    return ret

def anonymousRegister():
    s = xmlrpclib.Server(serverUrl, proxyHost)
    try:
        systemId = s.registration.anonymous(getVersion(), getArch())
    except KeyboardInterrupt:
        raise CommunicationError(_("Connection aborted by the user"))
    except (socket.error, socket.sslerror), e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        raise CommunicationError(f.faultString)

    path = cfg.readEntry("systemIdPath")
    dir = path[:string.rfind(path, "/")]
    if not os.access(dir, os.W_OK):
        try:
            os.mkdir(dir)
        except:
            return 0
    if not os.access(dir, os.W_OK):
        return 0

    f = open(path, "w")
    f.write(systemId)
    f.close()
    try:
        os.chmod(path, 0600)
    except:
        return 0
    return 1
    

def checkGPGInstallation():
    if not os.access("/usr/bin/gpg", os.X_OK):
        # not installed or accessible.
        return 1

    output = os.popen("/usr/bin/gpg --list-keys 2>/dev/null", "r").read()
    if not re.search("pub  1024D/DB42A60E 1999-09-23 Red Hat, Inc <security@redhat.com>", output):
        return 2

    return 0

def addGPGKey():
    return os.system("/usr/bin/gpg --import /usr/share/rhn/RPM-GPG-KEY >/dev/null 2>&1")

def retrieveOnly():
    return cfg.readEntry("retrieveOnly")
        
def installedHeader(someName):
    h = None

    db = rpm.opendb(0, "/")
    
    if type(someName) == type([]):
        pkgName = someName[0]
    else:
        pkgName = someName

    try:
        for index in db.findbyname(pkgName):
            h = db[index]
            break
    except rpm.error, e:
        raise RpmError(e)

    if h != None:
        name = h['name']
        epoch = h['epoch']
        if epoch == None:
            epoch = ""
        version = h['version']
        release = h['release']
        if type(someName) == type([]):
            if (pkgName == name and
                pkg[2] == version and
                pkg[3] == release and
                pkg[4] == epoch):
                return h
        else:
            if (pkgName == name):
                return h

    # not found
    return None

def pkgToString(pkg):
    return "%s-%s-%s" % (pkg[0], pkg[1], pkg[2])

# compare two RPM packages
def comparePackages(pkgLabel1, pkgLabel2):
    name1 = pkgLabel1[0]
    version1 = pkgLabel1[1]
    release1 = pkgLabel1[2]
    epoch1 = pkgLabel1[3]
    name2 = pkgLabel2[0]
    version2 = pkgLabel2[1]
    release2 = pkgLabel2[2]
    epoch2 = pkgLabel2[3]

    if epoch1 == "":
        epoch1 = None
    else:
        epoch1 = "%s" % epoch1
        
    if epoch2 == "":
            epoch2 = None
    else:
        epoch2 = "%s" % epoch2
        
    return rpm.labelCompare((epoch1, version1, release1),
                            (epoch2, version2, release2))


def md5(fileName):
    import md5
    hash = md5.new()
    
    try:
        f = open(fileName, "r")
    except:
        return ""

    fData = f.read()
    hash.update(fData)
    del fData
    f.close()
    
    hex = string.hexdigits
    md5 = ""
    for c in hash.digest():
        i = ord(c)
        md5 = md5 + hex[(i >> 4) & 0xF] + hex[i & 0xf]

    return md5
    
# checks if a package described by the header parameter
# has modified config files.  Returns a list of filenames
def modifiedConfigs(header):
    fileNames = header['fileNames']
    fileFlags = header['fileflags']
    md5Sums = header['filemd5s']
    modified = []
    
    for f_i in range(len(fileNames)):
        if fileFlags[f_i] & rpm.RPMFILE_CONFIG:
            if md5Sums[f_i] != md5(fileNames[f_i]):
                modified.append(fileNames[f_i])

    return modified

# returns a list of config filenames from a header
def configFiles(header):
    fileNames = header['filenames']
    fileFlags = header['fileflags']
    configs = []
    
    for f_i in range(len(fileNames)):
        if fileFlags[f_i] & rpm.RPMFILE_CONFIG:
            configs.append(fileNames[f_i])

    return configs


def getAvailablePackageList():
    availPkgList = []

    s = xmlrpclib.Server(serverUrl, proxyHost)
    
    try:
        availPkgList = s.up2date.listall(getSystemId())
    except KeyboardInterrupt:
        raise CommunicationError(_("Connection aborted by the user"))
    except (socket.error, socket.sslerror), e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])        
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        raise CommunicationError(f.faultString)

    return availPkgList


def getHeader(pkg):
    hdr = None

    # first see if it in memory cache:
    if headerCache.has_key(pkgToString(pkg)):
        return headerCache[pkgToString(pkg)]
        
    # next see if it is on disk:
    fileNames = glob.glob("%s/%s.%s.*" %
                          (cfg.readEntry("storageDir"),
                           pkgToString(pkg),
                           archGlob()))
    if len(fileNames) :
        if re.search("rpm$", fileNames[0]):
            fd = os.open(fileNames[0], os.O_RDONLY)
            try:
                hdr = rpm.headerFromPackage(fd)[0]
            except:
                raise RpmError(_("Error reading header from package"))
            os.close(fd)
        else:
            fd = open(fileNames[0], "r")
            hdr = rpm.headerLoad(fd.read())
            fd.close()
    else:
        fileNames = glob.glob("%s/%s.noarch.*" %
                              (cfg.readEntry("storageDir"),
                               pkgToString(pkg)))
        if len(fileNames) :
            if re.search("rpm$", fileNames[0]):
                fd = os.open(fileNames[0], os.O_RDONLY)
                hdr = rpm.headerFromPackage(fd)[0]
                os.close(fd)
            else:
                fd = open(fileNames[0], "r")
                hdr = rpm.headerLoad(fd.read())
                fd.close()
        else:
            s = xmlrpclib.Server(serverUrl, proxyHost)
            try:
                ret = s.up2date.header(getSystemId(), pkg)[0]
            except (socket.error, socket.sslerror), e:
                if len(e.args) > 1:
                    raise CommunicationError(e.args[1])
                else:
                    raise CommunicationError(e.args[0])
            except cgiwrap.ProtocolError, e:
                raise CommunicationError(e.errmsg)
            except xmlrpclib.Fault, f:
                raise CommunicationError(f.faultString)

            hdr = rpm.headerLoad(ret.data)
            saveHeader(hdr)

    return hdr


# implicitly places headers in the headerCache.
def getHeaders(pkgList, msgCallback = None, progressCallback = None):
    notOnDiskList = []

    for pkg in pkgList:
        hdr = None

        # first see if it is on disk:
        fileNames = glob.glob("%s/%s.%s.hdr" %
                              (cfg.readEntry("storageDir"),
                               pkgToString(pkg),
                               archGlob()))
        if len(fileNames) :
            fd = open(fileNames[0], "r")
            hdr = rpm.headerLoad(fd.read())
            fd.close()
            headerCache[pkgToString(pkg)] = hdr
        else:
            fileNames = glob.glob("%s/%s.noarch.hdr" %
                                  (cfg.readEntry("storageDir"),
                                   pkgToString(pkg)))
            if len(fileNames) :
                fd = open(fileNames[0], "r")
                hdr = rpm.headerLoad(fd.read())
                fd.close()
                headerCache[pkgToString(pkg)] = hdr
            else:
                notOnDiskList.append(pkg)


    # any remaining ones we need to get from the server
    if len(notOnDiskList):
        s = xmlrpclib.Server(serverUrl, proxyHost)
        try:
            ret = s.up2date.header(getSystemId(), notOnDiskList)
        except KeyboardInterrupt:
            raise CommunicationError(_("Connection aborted by the user"))
        except (socket.error, socket.sslerror), e:
            if len(e.args) > 1:
                raise CommunicationError(e.args[1])
            else:
                raise CommunicationError(e.args[0])
        except cgiwrap.ProtocolError, e:
            raise CommunicationError(e.errmsg)
        except xmlrpclib.Fault, f:
            raise CommunicationError(f.faultString)

        for bin in ret:
            hdr = rpm.headerLoad(bin.data)
            saveHeader(hdr)
            headerCache["%s-%s-%s" % (hdr['name'],
                                      hdr['version'],
                                      hdr['release'])] = hdr

    return 1


def solveDep(unknowns):
    retList = []
    
    s = xmlrpclib.Server(serverUrl, proxyHost)

    try:
        retList = s.up2date.solvedep(getSystemId(), unknowns)
    except KeyboardInterrupt:
        raise CommunicationError(_("Connection aborted by the user"))
    except (socket.error, socket.sslerror), e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == -6:
            raise RpmError(f.faultString + _(", depended on by %s") % unknowns)
        else:
            raise CommunicationError(f.faultString)

    return retList


def saveHeader(hdr):
    fileName = "%s/%s.%s.hdr" % (cfg.readEntry("storageDir"),
                              string.join((hdr['name'],
                                           hdr['version'],
                                           hdr['release']), "-"), hdr['arch'])

    try:
        fd = os.open(fileName, os.O_WRONLY|os.O_CREAT, 0600)
    except:
        print _("Error opening output file %s for writing.") % fileName
        return 0

    os.write(fd, hdr.unload())
    os.close(fd)

    return 1

    
def removeHeader(pkg):
    # first see if it is on disk:
    fileNames = glob.glob("%s/%s.%s.hdr" %
                          (cfg.readEntry("storageDir"),
                           pkgToString(pkg),
                           archGlob()))
    for fileName in fileNames:
        try:
            os.remove(fileName)
        except:
            raise FileError(_("cannot remove %s.") % fileName)

    fileNames = glob.glob("%s/%s.noarch.hdr" %
                          (cfg.readEntry("storageDir"),
                           pkgToString(pkg)))
    for fileName in fileNames:
        try:
            os.remove(fileName)
        except:
            raise FileError(_("cannot remove %s.") % fileName)


def getPackage(pkg, msgCallback = None, progressCallback = None):
    hdr = getHeader(pkg)

    if progressCallback != None:
        count = 0
        
    fileName = "%s.%s.rpm" % (string.join((hdr['name'],
                                           hdr['version'],
                                           hdr['release']), "-"),
                              hdr['arch'])

    if msgCallback != None:
        msgCallback("%s" % fileName)

    fileName = "%s/%s.%s.rpm" % (cfg.readEntry("storageDir"),
                                 string.join((hdr['name'],
                                              hdr['version'],
                                              hdr['release']), "-"),
                                 hdr['arch'])

    # check if we already have package
    if os.access(fileName, os.R_OK):
        fSize = os.stat(fileName)[6]
        if progressCallback != None:
            progressCallback(1, 1, 1)
    else:
        s = xmlrpclib.Server(serverUrl, proxyHost)
        try:
            pkgStream = s.up2date.package(getSystemId(), pkg)
        except KeyboardInterrupt:
            raise CommunicationError(_("Connection aborted by the user"))
        except (socket.error, socket.sslerror), e:
            if len(e.args) > 1:
                raise CommunicationError(e.args[1])
            else:
                raise CommunicationError(e.args[0])
        except cgiwrap.ProtocolError, e:
            raise CommunicationError(e.errmsg)
        except xmlrpclib.Fault, f:
            raise CommunicationError(f.faultString)

        try:
            fd = os.open(fileName, os.O_WRONLY|os.O_CREAT, 0600)
        except:
            print _("Error opening output file %s for writing.") % fileName
            return 0

        chunk = pkgStream.read(1024)
        while len(chunk) > 0:
            try:
                os.write(fd, chunk)
            except:
                os.close(fd)
                print _("Error writing to %s") % fileName
                return 0

            if progressCallback != None:
                count = count + len(chunk)
                progressCallback(count, len(pkgStream))

            chunk = pkgStream.read(1024)

        os.close(fd)

    if cfg.readEntry("retrieveSource"):
        s = xmlrpclib.Server(serverUrl, proxyHost)
        try:
            pkgStream = s.up2date.source_package_by_name(getSystemId(),
                                                         hdr['SOURCERPM'])
        except KeyboardInterrupt:
            raise CommunicationError(_("Connection aborted by the user"))
        except (socket.error, socket.sslerror), e:
            if len(e.args) > 1:
                raise CommunicationError(e.args[1])
            else:
                raise CommunicationError(e.args[0])
        except cgiwrap.ProtocolError, e:
            raise CommunicationError(e.errmsg)
        except xmlrpclib.Fault, f:
            raise CommunicationError(f.faultString)

        fileName = pkgStream.name
        if fileName == "":
            # if the database isn't properly populated we get an empty string
            return 1

        if msgCallback != None:
            msgCallback("%s" % fileName)

        fileName = "%s/%s" % (cfg.readEntry("storageDir"), fileName)

        # check if we already have package
        if os.access(fileName, os.R_OK):
            if progressCallback != None:
                progressCallback(1, 1)

            return 1

        try:
            fd = os.open(fileName, os.O_WRONLY|os.O_CREAT, 0600)
        except:
            print _("Error opening output file %s for writing.") % fileName
            return 0

        chunk = pkgStream.read(1024)
        while len(chunk) > 0:
            try:
                os.write(fd, chunk)
            except:
                os.close(fd)
                print _("Error writing to %s") % fileName
                return 0

            if progressCallback != None:
                count = count + len(chunk)
                progressCallback(count, len(pkgStream))

            chunk = pkgStream.read(1024)

        os.close(fd)

    return 1


def removePackage(pkg):
    # first see if it is on disk:
    fileNames = glob.glob("%s/%s.%s.*" %
                          (cfg.readEntry("storageDir"),
                           pkgToString(pkg),
                           archGlob()))
    for fileName in fileNames:
        try:
            os.remove(fileName)
        except:
            raise FileError(_("Cannot remove %s") % fileName)
    
    fileNames = glob.glob("%s/%s.noarch.*" %
                          (cfg.readEntry("storageDir"),
                           pkgToString(pkg)))
    for fileName in fileNames:
        try:
            os.remove(fileName)
        except:
            raise FileError(_("Cannot remove %s") % fileName)


def getPackageInfo(pkg):
    s = xmlrpclib.Server(serverUrl, proxyHost)
    try:
        info = s.up2date.description(getSystemId(), [pkg])
    except KeyboardInterrupt:
        raise CommunicationError(_("Connection aborted by the user"))
    except xmlrpclib.Fault, f:
        print f
        return None

    return info


def remoteAddPackages(pkg):
    s = xmlrpclib.Server(serverUrl, proxyHost)
    try:
        if type(pkg[0]) == type([]):
            #s.set_transport(2, 2)
            s.registration.add_packages(getSystemId(), pkg)
        else:
            s.registration.add_packages(getSystemId(), [pkg])            
    except KeyboardInterrupt:
        raise CommunicationError(_("Connection aborted by the user"))
    except (socket.error, socket.sslerror), e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        raise CommunicationError(f.faultString)


def remoteDelPackages(pkg):
    s = xmlrpclib.Server(serverUrl, proxyHost)
    try:
        if type(pkg[0]) == type([]):
            #s.set_transport(2, 2)
            s.registration.delete_packages(getSystemId(), pkg)
        else:
            s.registration.delete_packages(getSystemId(), [pkg])            
    except KeyboardInterrupt:
        raise CommunicationError(_("Connection aborted by the user"))
    except (socket.error, socket.sslerror), e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        raise CommunicationError(f.faultString)


def updatePackageProfile():
    s = xmlrpclib.Server(serverUrl, proxyHost)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])
    
    try:
        #s.set_transport(2, 2)
        s.registration.update_packages(getSystemId(),
                                       getInstalledPackageList())
    except KeyboardInterrupt:
        raise CommunicationError(_("Connection aborted by the user"))
    except (socket.error, socket.sslerror), e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)


def getInstalledPackageList(msgCallback = None, progressCallback = None):
    list = []

    if msgCallback != None:
        msgCallback(_("Getting list of packages installed on the system"))

    try:
        db = rpm.opendb(0, "/")
    except rpm.error, e:
        raise RpmError(_("Could not RPM database for reading.  Perhaps it is already in use?"))

    if progressCallback != None:
        count = 1
        total = 0

        if "firstkey" in dir(db):
            key = db.firstkey()
            h = db[key]
        else:
            iterator = db.match(0)
            h = iterator.next()
        while (h):
            count = count + 1
            if "nextkey" in dir(db):
                key = db.nextkey(key)
                if key:
                    h = db[key]
                else:
                    h = None
            else:
                h = iterator.next()

        total = count
        count = 1

    if "firstkey" in dir(db):
        key = db.firstkey()
        h = db[key]
    else:
        iterator = db.match(0)
        h = iterator.next()
    while (h):
        name = h['name']
        epoch = h['epoch']
        if epoch == None:
            epoch = ""
        version = h['version']
        release = h['release']

        list.append([name, version, release, epoch])
        if progressCallback != None:
            progressCallback(count, total)
            count = count + 1
        if "nextkey" in dir(db):
            key = db.nextkey(key)
            if key:
                h = db[key]
            else:
                h = None
        else:
            h = iterator.next()

    list.sort()
    return list

# side effects on pkgList
def removeInstalledPackagesFromList(pkgList, retrieveNew = 0,
                                    msgCallback = None,
                                    progressCallback = None):
    removedList = []
    pkgCount = len(pkgList)
    installedPkgList = getInstalledPackageList()

    if msgCallback != None:
        msgCallback(_("Removing installed packages from list of updates"))
        
    for index in range(pkgCount):
        for installedPkg in installedPkgList:
            if installedPkg[0] == pkgList[index][0]:
                ret = comparePackages(pkgList[index], installedPkg)
                if ret == 0:
                    # already installed
                    removedList.append(pkgList[index])
                elif ret < 0:
                    # newer version already installed
                    removedList.append(pkgList[index])
                else:
                    # we need to fetch it.
                    pass
                break
        else:
            if retrieveNew:
                # we need to fetch it.
                pass
            else:
                # not installed, we don't want it
                removedList.append(pkgList[index])

        if progressCallback:
            progressCallback(index, pkgCount)
            
    for pkg in removedList:
        pkgList.remove(pkg)

    return removedList
        
# side effects on pkgList
def removeSkipPackagesFromList(pkgList, msgCallback = None,
                               progressCallback = None):
    removedList = []
    pkgCount = len(pkgList)

    if msgCallback != None:
        msgCallback(_("Removing packages marked to skip from list"))
        
    for index in range(pkgCount):
        for pattern in cfg.readEntry("pkgSkipList"):
            if fnmatch.fnmatch(pkgList[index][0], pattern):
                removedList.append((pkgList[index], _("Pkg name/pattern")))
                break

        if progressCallback != None:
            progressCallback(index, pkgCount)



    for pkg in removedList:
        pkgList.remove(pkg[0])

    return removedList


# check to see if a new file has a different MD5 sum
# from the package already on disk, and if true, if
# the on-disk file has been modified.    
def checkModified(hdr, index):
    ret = 0
    fileName = hdr['filenames'][index]
    installedHdr = installedHeader(hdr['name'])
    if installedHdr == None:
        return 0

    MD5Res = ""
    installedFileNames = installedHdr['filenames']
    installedFileMD5s = installedHdr['filemd5s']

    for j in range(len(installedFileNames)):
        if (installedFileNames[j] == fileName):
            if installedFileMD5s[j] != hdr['filemd5s'][index]:
                #print "both new and old pkg have %s, with same MD5sum." % fileName
                pass
                
            else:
                #print "both new and old pkg have %s, but MD5sum has changed." % fileName
                if installedFileMD5s[j] != md5(fileName):
                    #print "file %s has changed on disk as well." % fileName
                    ret = 1
                else:
                    # not changed on disk
                    pass
            break

    return ret


# check if the signature matches the redhat.com one
def hasBadSignature(pkg):
    hdr = getHeader(pkg)
    fileName = "%s/%s.%s.rpm" % (cfg.readEntry("storageDir"),
                                 pkgToString(pkg),
                                 hdr['arch'])
    # must use the actual header from the real package.  The headers
    # the server returns by themselves have no signature info in them.
    fd = os.open(fileName, os.O_RDONLY)
    try:
        hdr = rpm.headerFromPackage(fd)[0]
    except:
        raise RpmError(_("Failed to read header from %s for signature verification") % fileName)
    
    os.close(fd)

    try:
        # only newer RPMs allow this
        if hdr['SIGGPG'] == None:
            return 1
        if "checksig" in dir(rpm):
            if rpm.checksig(fileName, rpm.RPMTAG_SIGGPG):
                return 2

        return 0
    except KeyError:
        # legacy 3.0.x RPM
        sigOk = 1
        lines = os.popen("/bin/rpm -Kv %s 2>&1" % fileName, "r").read()
        if re.search("BAD signature from \"Red Hat, Inc <security@redhat.com>\"", lines):
            sigOk = 2
        elif re.search("Good signature from \"Red Hat, Inc <security@redhat.com>\"", lines) and re.search("Fingerprint: CA20 8686 2BD6 9DFC 65F6  ECC4 2191 80CD DB42 A60E", lines):
            sigOk = 0

        return sigOk
            

# side effects on pkgList
def removeSkipFilesPackagesFromList(pkgList, msgCallback = None,
                                    progressCallback = None):
    removedList = []

    pkgCount = len(pkgList)

    if msgCallback != None:
        msgCallback(_("Removing packages with files marked to skip from list"))
        
    for p_i in range(pkgCount):
        if progressCallback != None:
            progressCallback(p_i, pkgCount)

        h = getHeader(pkgList[p_i])

        fflag = 0
        fileNames = h['filenames']
        for f_i in range(len(fileNames)):
            for pattern in cfg.readEntry("fileSkipList"):
                if fnmatch.fnmatch(fileNames[f_i], pattern):
                    removedList.append((pkgList[p_i], _("File name/pattern")))
                    fflag = 1
                    break
            # if we found a matching file, no need to
            # examine the rest in this package
            if fflag:
                break

            # Deal with config files
            if cfg.readEntry("noReplaceConfig"):
                if h['fileflags'][f_i] & rpm.RPMFILE_CONFIG:
                    # check if config file and if so, if modified
                    if checkModified(h, f_i):
                        removedList.append((pkgList[p_i], _("Config modified")))
                        fflag = 1
                        break

            if fflag:
                break

    for pkg in removedList:
        pkgList.remove(pkg[0])

    return removedList

# return a list of packages that need to be updated according
# to the preferences set in the config file, etc.
def getUpdatedPackageList(msgCallback = None, progressCallback = None):
    pkgSkipList = []
    
    if msgCallback != None:
        msgCallback(_("Retrieving list of all available packages"))

    pkgList = getAvailablePackageList()
    
    removeInstalledPackagesFromList(pkgList, cfg.readEntry("retrieveNew"),
                                    msgCallback, progressCallback)

    if len(pkgList) > 0:
        pkgSkipList = removeSkipPackagesFromList(pkgList, msgCallback,
                                                 progressCallback)

    if len(pkgList) > 0:
        if msgCallback != None:
            msgCallback(_("Getting headers for available packages"))
        total = len(pkgList)

        for index in range(total):
            hdr = getHeader(pkgList[index])
            headerCache[pkgToString(pkgList[index])] = hdr
            if progressCallback:
                progressCallback(index, total)

    if len(pkgList) > 0:
        pkgSkipList = pkgSkipList + removeSkipFilesPackagesFromList(pkgList, msgCallback,
                                                                    progressCallback)

    if len(pkgSkipList) > 0:
        if msgCallback != None:
            msgCallback(_("Getting headers for skipped packages"))
        total = len(pkgSkipList)

        for index in range(total):
            hdr = getHeader(pkgSkipList[index][0])
            headerCache[pkgToString(pkgSkipList[index][0])] = hdr
            if progressCallback:
                progressCallback(index, total)

    pkgList.sort()
    pkgSkipList.sort()
    return (pkgList, pkgSkipList)
    
def totalSize(pkgList):
    total = 0
    
    for pkg in pkgList:
        hdr = getHeader(pkg)
        if hdr:
            total = total + hdr['size']

    return total
    
def freeDiskSpace():
    import statvfs

    dfInfo = os.statvfs(cfg.readEntry("storageDir"))
    return long(dfInfo[statvfs.F_BAVAIL]) * (dfInfo[statvfs.F_BSIZE])


def rpmCallback(what, amount, total, key, cb):
    if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
        fd = os.open(key, os.O_RDONLY)
        return fd
    elif what == rpm.RPMCALLBACK_INST_START:
        pass
    elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
        print
    elif what == rpm.RPMCALLBACK_TRANS_PROGRESS:
        if cb:
            cb(amount, total)
        else:
            print "transaction %.5s%% done\r" % (((1.0 * amount) / total) * 100),
    elif what == rpm.RPMCALLBACK_INST_PROGRESS:
        print "installation %.5s%% done\r" % (((1.0 * amount) / total) * 100), 


def dryRun(pkgList, msgCallback = None, progressCallback = None):
    total = len(pkgList)
    newList = []
    
    if msgCallback != None:
        msgCallback(_("Testing package set / solving RPM inter-dependencies"))

    if progressCallback:
        progressCallback(0, 1)
    
    db = rpm.opendb(0, "/")
    ts = rpm.TransactionSet("/", db)

    for p_i in range(total):
        hdr = getHeader(pkgList[p_i])
        
        if hdr:
            ts.add(hdr, pkgList[p_i], "u")
        else:
            raise RpmError(_("Could not find header for %s") % pkgList[p_i])

    ts.order()
    deps = ts.depcheck()
    oldDeps = []

    if deps:
        # make the deps list unique:
        deps.sort()
        last = deps[-1]
        for i in range(len(deps)-2, -1, -1):
            if last == deps[i]:
                del deps[i]
            else:
                last = deps[i]
                
        # we will probably need information from this below, get it now.
        availPkgList = getAvailablePackageList()

    while deps:
        # process each dependency, and see if RHN can satisfy
        # it for us.  If so, add those packages which satisfy it to
        # the list of packages to upgrade.  If not, raise an error.
        for dep in deps:
            ((name, version, release),
             (needsName, needsVersion),
             flags, suggested, sense) = dep
            # info on flags:
            # 2 = less than
            # 4 = greater than
            # 8 = equal to


            if oldDeps != deps:
                if sense == rpm.RPMDEP_SENSE_REQUIRES:
                    solvedList = solveDep([needsName])
                else:
                    raise RpmError(_("Don't know how to handle dependency sense \"%s\"") % sense)

                if solvedList == None:
                    raise RpmError(_("Could not satisfy dependencies."))

                for pkg in solvedList:
                    if not pkg in pkgList:
                        hdr = getHeader(pkg)
                        if hdr:
                            ts.add(hdr, pkg, "u")
                            pkgList.append(pkg)
                            newList.append(pkg)
                        else:
                            raise RpmError(_("Could not find header for %s") % pkg)
            else:
                # see if we can solve the dependencies by upgrading
                # the package that is raising the dependency
                changed = 0
                for pkg in availPkgList:
                    if pkg[0] == name and not pkg in pkgList:
                        hdr = getHeader(pkg)
                        if hdr:
                            ts.add(hdr, pkg, "u")
                            pkgList.append(pkg)
                            newList.append(pkg)
                            changed = 1
                        else:
                            raise RpmError(_("Could not find header for %s") % pkg)
                if not changed:
                    raise RpmError(_("Unresolvable chain of dependencies: %s-%s-%s requires %s-%s.") % (name, version, release, needsName, needsVersion))


        # save a copy of the current dependency set for comparison
        # with next dependency set.
        oldDeps = deps
        ts.order()
        deps = ts.depcheck()

        # make the deps list unique:
        if deps:
            deps.sort()
            last = deps[-1]
            for i in range(len(deps)-2, -1, -1):
                if last == deps[i]:
                    del deps[i]
                else:
                    last = deps[i]        


    if progressCallback:
        progressCallback(1,1)

    rc = ts.run(rpm.RPMTRANS_FLAG_BUILD_PROBS, 0, rpmCallback, progressCallback)
    if len(rc):
        msg = ""
        for code in rc:
            msg = msg + code[0] + "\n"
        raise ConflictError(_("Test install failed because of package conflicts:\n") + msg)
    else:
        return newList

def installPackages(pkgList, rpmCallback):
    total = len(pkgList)

    oldInstalledPackages = getInstalledPackageList()
    
    # delete any existing db, open in write mode
    try:
        db = rpm.opendb(1, "/")
    except:
        raise RpmError(_("Failed to open RPM database for writing."))
    
    ts = rpm.TransactionSet("/", db)

    for p_i in range(total):
        hdr = getHeader(pkgList[p_i])
        if hdr:
            #fileName = "%s/%s.%s.rpm" % (cfg.readEntry("storageDir"),
            #                             pkgToString(pkgList[p_i]),
            #                             hdr['arch'])
            ts.add(hdr, hdr, "u")
        else:
            raise RpmError(_("Could not find header for %s") % pkgList[p_i])

    ts.order()
    deps = ts.depcheck()
    if deps:
        raise RpmError(_("Dependencies should have already been resolved, but they are not."))

    rc = ts.run(0, 0, rpmCallback, cfg.readEntry("storageDir"))
    if rc:
        raise RpmError(_("Failed installing packages:"), rc)

    newInstalledPackages = getInstalledPackageList()

    removedPackages = []
    addedPackages = []
    
    # find packages that were removed
    for pkg in oldInstalledPackages:
        if pkg not in newInstalledPackages:
            removedPackages.append(pkg)

    for pkg in newInstalledPackages:
        if pkg not in oldInstalledPackages:
            addedPackages.append(pkg)

    remoteDelPackages(removedPackages)    
    remoteAddPackages(addedPackages)

    return 1
    
def msgCallback(msg):
    print msg


def simpleCallback(count, total):
    print "%.5s%% done\r" % (((1.0 * count) / total) * 100),
